// Class ControlledGridAppFrame
//
// Author: Alyce Brady
//
// This class is based on the College Board's MBSGUIFrame class,
// as allowed by the GNU General Public License.  MBSGUIFrame
// is a black-box class within the AP(r) CS Marine Biology Simulation
// case study (see www.collegeboard.com/ap/students/compsci).
//
// License Information:
//   This class is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation.
//
//   This class is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

package edu.kzoo.grid.gui;

import edu.kzoo.grid.Grid;

import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.Timer;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 *  Grid GUI Support Package:<br>
 *
 *  The <code>ControlledGridAppFrame</code> class provides a window in which
 *  to run and display a grid application controlled by the user via
 *  a combination of Step, NSteps, Run, and Stop buttons.  Other options
 *  include a speed slider bar and a Start/Restart button.
 *
 *  @author Alyce Brady (based on code by Julie Zelenski)
 *  @version 29 February 2004
 **/
public class ControlledGridAppFrame extends GridAppFrame
{
    protected GridAppController appController;
    protected Timer   timer;
    protected boolean putRestartFirst;
    protected JButton stepButton, nStepsButton,
                      runButton, stopButton;
    protected boolean includeStep, includeNSteps, includeRun, includeStop;
    protected boolean running, runningNSteps;
    protected int     numStepsToRun, numStepsSoFar;


  // constructors and initialization methods

    /** Constructs an empty ControlledGridAppFrame window object that will
     *  display a grid controlled by a combination of Step, NSteps, Run, and
     *  Stop buttons.  The boolean parameters determine which buttons
     *  will be included in the control panel of the graphical user
     *  interface.  Use methods such as <code>includeStartRestart</code>
     *  and <code>includeSpeedSlider</code> to include components on the
     *  other than the basic grid display and the specified run buttons.
     *  Then use the constructWindowContents method to set the properties
     *  of the window and make it visible.
     *  (Precondition: control is not <code>null</code>.)
     *    @param control            the object that controls the running of
     *                              the grid application through step
     *                              and/or run methods.
     *    @param includeStep        <code>true</code> if GUI should include
     *                              a One Step button
     *    @param includeNSteps      <code>true</code> if GUI should include
     *                              an N Steps button
     *    @param includeRun         <code>true</code> if GUI should include
     *                              a Run... button
     *    @param includeStop        <code>true</code> if GUI should include
     *                              a Stop button
     **/
    public ControlledGridAppFrame(GridAppController control,
                           boolean includeStep, boolean includeNSteps,
                           boolean includeRun, boolean includeStop)
    {
        appController = control;

        this.includeStep = includeStep;
        this.includeNSteps = includeNSteps;
        this.includeRun = includeRun;
        this.includeStop = includeStop;

        // Construct the buttons whether they will be included in the
        // control panel or not, so that the enable/disable code does not
        // have to constantly check for each one. 
        stepButton = new JButton("Step Once");
        stepButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e)
                { step(); }});

        nStepsButton = new JButton("Step N Times");
        nStepsButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e)
                { nSteps(); }});

        runButton = new JButton("Run...");
        runButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e)
                { run(); }});

        stopButton = new JButton("Stop");
        stopButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e)
                { stop(); }});
    }

    /** Includes a Start/Restart/Reset button with the specified label.
     *  This method will have no effect unless it is
     *  called before the constructWindowContents method.
     *     @param label the button label (examples: "Start", "Reset") 
     *     @param enabled true if button should initially be enabled;
     *                    false if button should initially be disabled
     **/
    public void includeStartRestart(String label, boolean enabled)
    {
        includeRestartAfterRunButtons(label, enabled);
    }

    /** Includes a Start/Restart/Reset button with the specified label
     *  before the step, N steps, run, and stop buttons.
     *  This method will have no effect unless it is
     *  called before the constructWindowContents method.
     *     @param label the button label (examples: "Start", "Reset") 
     *     @param enabled true if button should initially be enabled;
     *                    false if button should initially be disabled
     **/
    public void includeRestartBeforeRunButtons(String label, boolean enabled)
    {
        super.includeStartRestart(label, enabled);
        putRestartFirst = true;
    }

    /** Includes a Start/Restart/Reset button with the specified label
     *  after the step, N steps, run, and stop buttons.
     *  This method will have no effect unless it is
     *  called before the constructWindowContents method.
     *     @param label the button label (examples: "Start", "Reset") 
     *     @param enabled true if button should initially be enabled;
     *                    false if button should initially be disabled
     **/
    public void includeRestartAfterRunButtons(String label, boolean enabled)
    {
        super.includeStartRestart(label, enabled);
        putRestartFirst = false;
    }


  // accessor methods

    /** Returns the controller used to drive the application. **/
    public GridAppController getController()
    {
        return appController;
    }


  // methods and nested classes for building the control panel

    /** Creates the control panel.
     *    @param label description for this set of control components
     *    @return a panel containing the control components
     **/
    protected JPanel makeControlPanel()
    {
        // Create a panel for the control components.
        JPanel controlPanel = super.makeControlPanel();
        controlPanel.setBorder(BorderFactory.createTitledBorder("Control Buttons"));
        return controlPanel;
    }

    /** Creates the interior component for the control panel.  The actual
     *  buttons included in the panel depend on the values specified when this
     *  active grid window was constructed.
     *    @return a component representing the contents of the control panel
     **/
    protected JComponent makeControlPanelInterior()
    {
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(0, 1));

        if ( getResetButton() != null && putRestartFirst )
            buttonPanel.add(getResetButton());

        if ( includeStep )
            buttonPanel.add(stepButton);

        if ( includeNSteps )
            buttonPanel.add(nStepsButton);

        if ( includeRun )
            buttonPanel.add(runButton);

        if ( includeStop )
            buttonPanel.add(stopButton);

        if ( buttonPanel.getComponentCount() == 0 )
            return null;

        if ( getResetButton() != null && ! putRestartFirst )
            buttonPanel.add(getResetButton());

        stepButton.setEnabled(false);
        nStepsButton.setEnabled(false);
        runButton.setEnabled(false);
        stopButton.setEnabled(false);

        // Define timer used by NSteps and Run buttons.  Define it here
        // rather than in the constructor because it is critical that
        // the speed slider (if there's going to be one) be created first.
        timer = new Timer(getDelay(),
                          new ActionListener()
                          {   public void actionPerformed(ActionEvent evt)
                              {   step();   }
                          });


        // Timer needs to listen for changes to the speed slider.
        if ( getSpeedSlider() != null )
        {
            getSpeedSlider().addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent evt) {
                    timer.setDelay(((JSlider)evt.getSource()).getValue());
                }});
        }

        return buttonPanel;
    }


  // redefinition of methods from the GridDisplay interface

    /** Sets the Grid being displayed.
     *    @param grid the Grid to display
     **/
    public void setGrid(Grid grid)
    {
        if ( running )
            stop();
        super.setGrid(grid);

        // Set the application controller's grid to match this one.
        appController.setGrid(grid);

        // Enable and disable buttons as appropriate.
        if ( grid != null )
            enterNotRunningMode();
    }

    /** Shows the grid.
     **/
    public void showGrid()
    {
        getDisplay().showGrid();
    }


  // methods that implement button actions

    /** Advances the application one step.  Should be redefined if the
     *  application wants to show the grid after each step.  For example:
     *  <pre>
     *     super.step();
     *     showGrid();
     *  </pre>
     **/
    public void step()
    {
        appController.step();

        if ( runningNSteps )
            numStepsSoFar++;

        if ( running && shouldStop() )
            stop();
    }

    /** Determines whether a running application has reached
     *  a desired stopping state.
     *    @return <code>true</code> if the application should
     *            stop 
     **/
    public boolean shouldStop()
    { 
        return ( ( runningNSteps && numStepsSoFar == numStepsToRun ) ||
                 appController.hasReachedStoppingState() );
    }

    /** Advances the application multiple (N) steps (where N is provided
     *  by the user in a dialog box).  It may stop before the N steps
     *  have completed if the user clicks on the stop button (if one is
     *  provided) or if the application controller indicates that the
     *  application has reached a stopping state.
     **/
    public void nSteps()
    { 
        if ( promptUserForStepCount() )
        {
            runningNSteps = true;
            numStepsSoFar = 0;
            enterRunningMode();
            timer.start();
        }
    }

    /** Runs a dialog asking the user to input a number of steps
     *  for the run behavior. If successful, updates numStepsToRun.
     *  and returns true, else returns false.
     **/
    private boolean promptUserForStepCount() 
    {
        int suggested = numStepsToRun > 0 ? numStepsToRun : 10;
        String response = getInitialResponse(suggested);
        while (response != null)
            try
            {
                int userEntered = Integer.parseInt(response.trim());
                if (userEntered <= 0)
                    throw new NumberFormatException();
                numStepsToRun = userEntered;
                return true;
            } 
            catch (NumberFormatException ex)
            {   response = getClarificationResponse(suggested);   }
            catch (Exception ex)
            {   Toolkit.getDefaultToolkit().beep();   }
        return false;
    }

    /** Provides inital prompt for number of steps and gets result. **/
    protected String getInitialResponse(int suggested)
    {
        return (String)JOptionPane.showInputDialog(this, 
                        "Enter number of steps:",
                        "Input", JOptionPane.QUESTION_MESSAGE, null, null,
                        "" + suggested);
    }

    /** Provides a follow-up prompt for number of steps if initial response
     *  failed validation, and gets result.
     **/
    protected String getClarificationResponse(int suggested)
    {
        return (String)JOptionPane.showInputDialog(this, 
                        "Number of steps must be a valid, positive integer:",
                         "Input", JOptionPane.QUESTION_MESSAGE, null, null,
                         "" + suggested);
    }

    /** Starts a timer to repeatedly step the application at
     *  the speed currently indicated by the speed slider.
     *  It will stop when the user clicks on the stop button or
     *  when the application controller indicates that the application
     *  has reached a stopping state.
     **/
    public void run() 
    {
        runningNSteps = false;
        enterRunningMode();
        timer.start(); 
    }

    /** Stops any existing timer currently stepping the application.
     **/
    public void stop()
    {
        timer.stop();
        enterNotRunningMode();
    }

    /** Restarts the application.  Should be redefined if the
     *  application wants to show the grid after the restart.
     **/
    public void restart()
    {
        appController.restart();
        enterNotRunningMode();
    }

    /** Enables and disables GUI components as necessary while the application
     *  is running.
     **/
    protected void enterRunningMode()
    {
        running = true;
        if ( getDisplay() != null )
            getDisplay().setToolTipsEnabled(false); // hide tool tips while running

        stopButton.setEnabled(true);

        stepButton.setEnabled(false);
        nStepsButton.setEnabled(false);
        runButton.setEnabled(false);
        if ( getResetButton() != null )
            getResetButton().setEnabled(false);

        if ( getJMenuBar() != null )
            getJMenuBar().setEnabled(false);
    }

    /** Enables and disables GUI components as necessary when an application
     *  is not running.
     **/
    protected void enterNotRunningMode()
    {
        running = runningNSteps = false;
        if ( getDisplay() != null )
            getDisplay().setToolTipsEnabled(true);

        stopButton.setEnabled(false);

        stepButton.setEnabled(true); 
        nStepsButton.setEnabled(true);
        runButton.setEnabled(true); 
        if ( getResetButton() != null )
            getResetButton().setEnabled(true);

        if ( getJMenuBar() != null )
            getJMenuBar().setEnabled(true);
    }

}